/* global Bdf,Ficheform,$$ */

Ficheform.CorpusInclude = {};

Ficheform.CorpusInclude.init = function () {
    Bdf.$({ficheformEntry: "corpus-include", ficheformMode: "check"}).each(function (index, element) {
        _initCheckCorpus(element);
    });
    Bdf.$({ficheformEntry: "corpus-include", ficheformMode: "table"}).each(function (index, element) {
        _initTableCorpus(element);
    });
    _initLiage();
   
    
    function _initCheckCorpus(element) {
        let $entry = $(element);
        let entryKey = element.dataset.ficheformKey;
        let subsetName =  Bdf.getSubsetName($entry);
        let inputId = Bdf.$($entry, {ficheformRole: "textinput"})[0].id;
        let buttonId = Bdf.generateId();
        let createButtonId = Bdf.generateId();
        Bdf.$children($entry, {ficheformRole: "fiche-input"}).append(Bdf.render("ficheform:pioche", {
            buttonId: buttonId,
            action: "action-ficheform-Fiche",
            locKey: "_ link.pioche.fiche_one",
            locValue: Bdf.Loc.get("subset:corpus_" + subsetName),
            noTab: Ficheform.getOption("noTab"),
            createButtonId: createButtonId
        }));
        $$(buttonId).click(function () {
            Ficheform.CorpusInclude.pioche(entryKey, subsetName, inputId, true);
        });
        if (Bdf.$($entry, {ficheformRole: "poids"}).length > 0) {
            Ficheform.initPoidsInput($entry);
        }
    }
    
    function _initTableCorpus(element) {
        let $entry = $(element);
        let entryKey = element.dataset.ficheformKey;
        let subsetName =  Bdf.getSubsetName($entry);
        let createButtonId = Bdf.generateId();
        let parentCorpus = Ficheform.getCurrentCorpus($entry);
        Bdf.$children($entry, {ficheformRole: "fiche-table"}).append(Bdf.render("ficheform:fichetable/tablebuttons", {
            noTab: Ficheform.getOption("noTab"),
            createButtonId: createButtonId
        }));
        let $ficheRows = Bdf.$($entry, {ficheformRole: "fiche-rows"});
        let colNameArray = $ficheRows[0].dataset.ficheformColDefList.split(",");
        Bdf.$($ficheRows, {ficheformRole: "fiche-row"}).each(function (index, element) {
           __initRow(element); 
        });
        $$(createButtonId).click(function () {
            Bdf.Overlay.showFicheEditor(subsetName, -1, {
                doneCallback: function (subsetName, id) {
                    Bdf.Ajax.loadFiche(subsetName, id, "_table", parentCorpus, function (fiche) {
                        let cellArray = [];
                        for(let colName of colNameArray) {
                            if (fiche.properties.hasOwnProperty(colName)) {
                                cellArray.push(fiche.properties[colName]);
                            } else {
                                cellArray.push("");
                            }
                        }
                         $ficheRows.append(Bdf.render("ficheform:fichetable/ficherow", {
                            checkboxName: entryKey,
                            checkboxValue: id,
                            cellArray: cellArray
                        }));
                        __initRow(Bdf.$($ficheRows, {ficheformFicheId: id})[0]);
                        Ficheform.signalChange(true);
                    });
                }
            });
        });
        
        function __initRow(tr) {
            let ficheId = tr.dataset.ficheformFicheId;
            let genId = Bdf.generateId();
            Bdf.$(tr, {ficheformRole: "fiche-row-buttons"}).html(Bdf.render("ficheform:fichetable/rowbuttons", {
                genId: genId,
                noTab: Ficheform.getOption("noTab")
               }));
            $$(genId, "up").click(function () {
                let $previous = $(tr).prev();
                if ($previous.length > 0) {
                    $previous.before(tr);
                    Ficheform.signalChange(true);
                }
            });
            $$(genId, "down").click(function () {
                let $next = $(tr).next();
                if ($next.length > 0) {
                    $next.after(tr);
                    Ficheform.signalChange(true);
                }
            });
            $$(genId, "edit").click(function () {
                Bdf.Overlay.showFicheEditor(subsetName, ficheId, {
                    doneCallback: function (subsetName, id) {
                        Bdf.Ajax.loadFiche(subsetName, id, "_table", parentCorpus, function (fiche) {
                            let cellArray = [];
                            for(let colName of colNameArray) {
                                if (fiche.properties.hasOwnProperty(colName)) {
                                    cellArray.push(fiche.properties[colName]);
                                } else {
                                    cellArray.push("");
                                }
                            }
                            Bdf.$($ficheRows, {ficheformFicheId: id}).children().each(function (index, element) {
                                if (index > 0) {
                                    $(element).html(cellArray[index - 1]);
                                }
                            });
                        });
                    }
                });
            });
            $$(genId, "remove").click(function () {
                
            });
        }
    }
    
    function _initLiage() {
        Bdf.$({ficheformEntry: "liage"}).each(function (index, element) {
            let $entry = $(element);
            let entryKey = element.dataset.ficheformKey;
            let inputId = Bdf.$($entry, {ficheformRole: "textinput"})[0].id;
            let buttonId = Bdf.generateId();
            Bdf.$children($entry, {ficheformRole: "fiche-input"}).append(Bdf.render("ficheform:pioche", {
                buttonId: buttonId,
                action: "action-ficheform-Liage",
                locKey: "_ link.pioche.fiche_all",
                noTab: Ficheform.getOption("noTab")
            }));
            $$(buttonId).click(function () {
                Ficheform.CorpusInclude.pioche(entryKey, "", inputId, false);
            });
            Ficheform.initPoidsInput($entry);
        });
    }

};

Ficheform.CorpusInclude.pioche = function (entryKey, subsets, inputId, withFicheCreationButton) {
    var $entry = Ficheform.$entry(entryKey);
    var limit = $entry.data("ficheformLimit");
    if (!limit) {
        limit = -1;
    }
    var url = "pioche?page=fiche&subsets=" + subsets + "&appelant=" + inputId + "&limit=" + limit;
    if (withFicheCreationButton) {
        url = url + "&buttons=fichecreation" ;
    }
    Ficheform.openDialog(url,"pioche_" + entryKey, 430,510);
};


Ficheform.CorpusInclude.showFicheCreation = function (inputId) {
    let $parent = Bdf.$parents($$(inputId), {subsetName: true});
    if ($parent.length > 0) {
        let subsetName = Bdf.getSubsetName($parent);
        Bdf.Overlay.showFicheEditor(subsetName, -1, {
            doneCallback: function (subsetName, id) {
                Bdf.appendValue(ID(inputId), id);
            }
        });
    }
};
